#!/bin/bash

set -u
set -e

declare -A orbits
declare -A reverse_orbits

exec 3<input.txt

while read -u 3 line; do
    base_object=${line%)*}
    orbit_object=${line#*)}

    if [ ${orbits[$base_object]+a} ]; then
        orbits[$base_object]+=",$orbit_object"
    else
        orbits[$base_object]=$orbit_object
    fi

    if [ ${reverse_orbits[$orbit_object]+a} ]; then
        reverse_orbits[$orbit_object]+=",$base_object"
    else
        reverse_orbits[$orbit_object]=$base_object
    fi
done

get_orbits() {
    planet=$1
    indirect=$2
    local total_orbits=0

    IFS=","
    if [ ${orbits[$planet]+a} ]; then
        for p in ${orbits[$planet]}; do
            o=$(get_orbits $p $((indirect+1)))
            total_orbits=$((total_orbits+$o+$indirect))
        done
    fi
    echo $total_orbits
}

get_path() {
    start_point=$1
    end_point=$2
    last_start=$3
    local offset=${4:-0}
    local path=${5:-$start_point}
    local shortest_path=-1
    local shortest_path_text=""

    local got_path=0

    IFS=","
    # check forwards
    if [ ${orbits[$start_point]+a} ]; then
        for p in ${orbits[$start_point]}; do
            if [ "$p" == "$last_start" ]; then
                continue
            fi
            if [ $p == $end_point ]; then
                shortest_path=$offset
                shortest_path_text="$path $p"
                echo $((shortest_path - 1))
                exit 0
            else
                o="$(get_path $p $end_point $start_point $((offset+1)) "$path $p")"
                exit_code=$?
                if [ $exit_code -eq 0 ]; then
                    if [ $shortest_path -eq -1 ] || [ $o -lt $shortest_path ]; then
                        shortest_path=$o
                        shortest_path_text="$path $p"
                        got_path=1
                    fi
                fi
            fi
        done
    fi

    if [ $got_path -eq 1 ]; then
        echo $shortest_path
        exit 0
    fi

    # otherwise, time to check other paths
    if [ ${reverse_orbits[$start_point]+a} ]; then
        for p in ${reverse_orbits[$start_point]}; do
            if [ "$p" == "$last_start" ]; then
                continue
            fi
            if [ "$p" == "$end_point" ]; then
                shortest_path=$offset
                shortest_path_text="$path $p"
                echo $((shortest_path - 1))
                exit 0
            else
                o="$(get_path $p $end_point $start_point $((offset+1)) "$path $p")"
                exit_code=$?
                if [ $exit_code -eq 0 ]; then
                    if [ $shortest_path -eq -1 ] || [ $o -lt $shortest_path ]; then
                        shortest_path=$o
                        shortest_path_text="$path $p"
                        got_path=1
                    fi
                fi
            fi
        done
    fi

    if [ $got_path -eq 1 ]; then
        echo $shortest_path
        got_path=0
        exit 0
    fi

    exit 1
}

echo -n "Orbits: "
get_orbits COM 1

shortest_path=-1
path=$(get_path YOU SAN "")
echo "Shortest path from YOU -> SAN: $path"
